@@ -1,8 +1,10 @@
+import os
 import pathlib
 import shutil
 import time
 import typing as t
 
+import toml
 import typer
 from rich import box, print
 from rich.live import Live
@@ -16,20 +18,27 @@ from dreadnode_cli.agent.docker import get_registry
 from dreadnode_cli.agent.format import (
     format_agent,
     format_agent_versions,
-    format_models,
     format_run,
+    format_run_groups,
     format_runs,
+    format_strike_models,
     format_strikes,
-    format_templates,
 )
-from dreadnode_cli.agent.templates import Template, install_template, install_template_from_dir
+from dreadnode_cli.agent.templates import cli as templates_cli
+from dreadnode_cli.agent.templates.format import format_templates
+from dreadnode_cli.agent.templates.manager import TemplateManager
+from dreadnode_cli.api import Client
 from dreadnode_cli.config import UserConfig
+from dreadnode_cli.model.config import UserModels
+from dreadnode_cli.model.format import format_user_models
 from dreadnode_cli.profile.cli import switch as switch_profile
 from dreadnode_cli.types import GithubRepo
-from dreadnode_cli.utils import download_and_unzip_archive, pretty_cli, repo_exists
+from dreadnode_cli.utils import download_and_unzip_archive, get_repo_archive_source_path, pretty_cli
 
 cli = typer.Typer(no_args_is_help=True)
 
+cli.add_typer(templates_cli, name="templates", help="Manage Agent templates")
+
 
 def ensure_profile(agent_config: AgentConfig, *, user_config: UserConfig | None = None) -> None:
     """Ensure the active agent link matches the current server profile."""
@@ -44,8 +53,8 @@ def ensure_profile(agent_config: AgentConfig, *, user_config: UserConfig | None
         plural = "s" if len(agent_config.linked_profiles) > 1 else ""
         raise Exception(
             f"This agent is linked to the [magenta]{linked_profiles}[/] server profile{plural}, "
-            f"but the current server profile is [yellow]{user_config.active_profile_name}[/], ",
-            "use [bold]dreadnode agent push[/] to create a new link with this profile.",
+            f"but the current server profile is [yellow]{user_config.active_profile_name}[/], "
+            "use [bold]dreadnode agent push[/] to create a new link with this profile."
         )
 
     if agent_config.active_link.profile != user_config.active_profile_name:
@@ -60,33 +69,15 @@ def ensure_profile(agent_config: AgentConfig, *, user_config: UserConfig | None
         ):
             print()
             raise Exception(
-                "Agent link does not match the current server profile. Use [bold]dreadnode agent switch[/] or [bold]dreadnode profile switch[/]."
+                f"Current agent link ([yellow]{agent_config.active_link.profile}[/]) does not match "
+                f"the current server profile ([magenta]{user_config.active_profile_name}[/]). "
+                "Use [bold]dreadnode agent switch[/] or [bold]dreadnode profile switch[/]."
             )
 
         switch_profile(agent_config.active_link.profile)
 
 
-def get_repo_archive_source_path(source_dir: pathlib.Path) -> pathlib.Path:
-    """Return the actual source directory from a git repositoryZIP archive."""
-
-    if not (source_dir / "Dockerfile").exists() and not (source_dir / "Dockerfile.j2").exists():
-        # if src has been downloaded from a ZIP archive, it may contain a single
-        # '<user>-<repo>-<commit hash>' folder, that is the actual source we want to use.
-        # Check if source_dir contains only one folder and update it if so.
-        children = list(source_dir.iterdir())
-        if len(children) == 1 and children[0].is_dir():
-            source_dir = children[0]
-
-    return source_dir
-
-
-@cli.command(help="List available agent templates with their descriptions")
-@pretty_cli
-def templates() -> None:
-    print(format_templates())
-
-
-@cli.command(help="Initialize a new agent project")
+@cli.command(help="Initialize a new agent project", no_args_is_help=True)
 @pretty_cli
 def init(
     strike: t.Annotated[str, typer.Argument(help="The target strike")],
@@ -98,8 +89,8 @@ def init(
         str | None, typer.Option("--name", "-n", help="The project name (used for container naming)")
     ] = None,
     template: t.Annotated[
-        Template, typer.Option("--template", "-t", help="The template to use for the agent")
-    ] = Template.rigging_basic,
+        str | None, typer.Option("--template", "-t", help="The template to use for the agent")
+    ] = None,
     source: t.Annotated[
         str | None,
         typer.Option(
@@ -137,18 +128,45 @@ def init(
     print(f":crossed_swords: Linking to strike '{strike_response.name}' ({strike_response.type})")
     print()
 
-    project_name = Prompt.ask("Project name?", default=name or directory.name)
+    project_name = Prompt.ask(":toolbox: Project name?", default=name or directory.name)
     print()
 
     directory.mkdir(exist_ok=True)
 
+    template_manager = TemplateManager()
     context = {"project_name": project_name, "strike": strike_response}
 
     if source is None:
-        # initialize from builtin template
-        template = Template(Prompt.ask("Template?", choices=[t.value for t in Template], default=template.value))
+        # get the templates that match the strike
+        available_templates = template_manager.get_templates_for_strike(strike_response.name, strike_response.type)
+        available: list[str] = list(available_templates.keys())
 
-        install_template(template, directory, context)
+        # none available
+        if not available:
+            if not template_manager.templates:
+                raise Exception(
+                    "No templates installed, use [bold]dreadnode agent templates install[/] to install some."
+                )
+            else:
+                raise Exception("No templates found for the given strike.")
+
+        # ask the user if the template has not been passed via command line
+        if template is None:
+            print(":notebook: Compatible templates:\n")
+            print(format_templates(available_templates, with_index=True))
+            print()
+
+            choice = Prompt.ask("Choice ", choices=[str(i + 1) for i in range(len(available))])
+            template = available[int(choice) - 1]
+
+        # validate the template
+        if template not in available:
+            raise Exception(
+                f"Template '{template}' not found, use [bold]dreadnode agent templates show[/] to see available templates."
+            )
+
+        # install the template
+        template_manager.install(template, directory, context)
     else:
         source_dir = pathlib.Path(source)
         cleanup = False
@@ -162,7 +180,7 @@ def init(
                 github_repo = GithubRepo(source)
 
                 # Check if the repo is accessible
-                if repo_exists(github_repo):
+                if github_repo.exists:
                     source_dir = download_and_unzip_archive(github_repo.zip_url)
 
                 # This could be a private repo that the user can access
@@ -193,7 +211,8 @@ def init(
             if path is not None:
                 source_dir = source_dir / path
 
-            install_template_from_dir(source_dir, directory, context)
+            # install the template
+            template_manager.install_from_dir(source_dir, directory, context)
         except Exception:
             if cleanup and source_dir.exists():
                 shutil.rmtree(source_dir)
@@ -232,7 +251,14 @@ def push(
 
     if agent_config.links and not agent_config.has_link_to_profile(user_config.active_profile_name):
         print(f":link: Linking as a fresh agent to the current profile [magenta]{user_config.active_profile_name}[/]")
+        print()
         new = True
+    elif agent_config.active and agent_config.active_link.profile != user_config.active_profile_name:
+        raise Exception(
+            f"Current agent link ([yellow]{agent_config.active_link.profile}[/]) does not match "
+            f"the current server profile ([magenta]{user_config.active_profile_name}[/]). "
+            "Use [bold]dreadnode agent switch[/] or [bold]dreadnode profile switch[/]."
+        )
 
     server_config = user_config.get_server_config()
 
@@ -244,7 +270,22 @@ def push(
     print()
     print(f":wrench: Building agent from [b]{directory}[/] ...")
     image = docker.build(directory, force_rebuild=rebuild)
-    repository = f"{registry}/{server_config.username}/agents/{agent_config.project_name}"
+    agent_name = docker.sanitized_name(agent_config.project_name)
+    sanitized_user_name = docker.sanitized_name(server_config.username)
+
+    if not agent_name:
+        raise Exception("Failed to sanitize agent name, please use a different name")
+
+    elif agent_name != agent_config.project_name:
+        print(f":four_leaf_clover: Agent name normalized to [bold magenta]{agent_name}[/]")
+
+    if not sanitized_user_name:
+        raise Exception("Failed to sanitize username")
+
+    elif sanitized_user_name != server_config.username:
+        print(f":four_leaf_clover: Username normalized to [bold magenta]{sanitized_user_name}[/]")
+
+    repository = f"{registry}/{sanitized_user_name}/agents/{agent_name}"
     tag = tag or image.id[-8:]
 
     print()
@@ -288,6 +329,31 @@ def push(
     print(":tada: Agent pushed. use [bold]dreadnode agent deploy[/] to start a new run.")
 
 
+def prepare_run_context(
+    env_vars: list[str] | None, parameters: list[str] | None, command: str | None
+) -> Client.StrikeRunContext | None:
+    if not env_vars and not parameters and not command:
+        return None
+
+    context = Client.StrikeRunContext()
+
+    if env_vars:
+        context.environment = {env_var.split("=")[0]: env_var.split("=")[1] for env_var in env_vars}
+
+    if parameters:
+        context.parameters = {}
+        for param in parameters:
+            if param.startswith("@"):
+                context.parameters.update(toml.load(open(param[1:])))
+            else:
+                context.parameters.update(toml.loads(param))
+
+    if command:
+        context.command = command
+
+    return context
+
+
 @cli.command(help="Start a new run using the latest active agent version")
 @pretty_cli
 def deploy(
@@ -298,12 +364,32 @@ def deploy(
         pathlib.Path,
         typer.Option("--dir", "-d", help="The agent directory", file_okay=False, resolve_path=True),
     ] = pathlib.Path("."),
+    env_vars: t.Annotated[
+        list[str] | None,
+        typer.Option("--env-var", "-e", help="Environment vars to override for this run (key=value)"),
+    ] = None,
+    parameters: t.Annotated[
+        list[str] | None,
+        typer.Option(
+            "--param",
+            "-p",
+            help="Define custom parameters for this run (key = value in toml syntax or @filename.toml for multiple values)",
+        ),
+    ] = None,
+    command: t.Annotated[
+        str | None,
+        typer.Option("--command", "-c", help="Override the container command for this run."),
+    ] = None,
     strike: t.Annotated[str | None, typer.Option("--strike", "-s", help="The strike to use for this run")] = None,
     watch: t.Annotated[bool, typer.Option("--watch", "-w", help="Watch the run status")] = True,
+    group: t.Annotated[str | None, typer.Option("--group", "-g", help="Group to associate this run with")] = None,
 ) -> None:
     agent_config = AgentConfig.read(directory)
     ensure_profile(agent_config)
 
+    user_config = UserConfig.read()
+    server_config = user_config.get_server_config()
+
     active_link = agent_config.active_link
 
     client = api.create_client()
@@ -313,16 +399,43 @@ def deploy(
     if strike is None:
         raise Exception("No strike specified, use -s/--strike or set the strike in the agent config")
 
-    # Verify the model if it was supplied
-    if model is not None:
+    context = prepare_run_context(env_vars, parameters, command)
+
+    user_models = UserModels.read()
+    user_model: Client.UserModel | None = None
+
+    # Check for a user-defined model
+    if model in user_models.models:
+        user_model = Client.UserModel(
+            key=model,
+            generator_id=user_models.models[model].generator_id,
+            api_key=user_models.models[model].api_key,
+        )
+
+        # Resolve the API key from env vars
+        if user_model.api_key.startswith("$"):
+            try:
+                user_model.api_key = os.environ[user_model.api_key[1:]]
+            except KeyError as e:
+                raise Exception(
+                    f"API key cannot be read from '{user_model.api_key}', environment variable not found."
+                ) from e
+
+    # Otherwise we'll ensure this is a valid strike-native model
+    if user_model is None and model is not None:
         strike_response = client.get_strike(strike)
         if not any(m.key == model for m in strike_response.models):
-            print(format_models(strike_response.models))
-            raise Exception(f"Model '{model}' not found in strike '{strike_response.name}'")
+            models(directory, strike=strike)
+            print()
+            raise Exception(
+                f"Model '{model}' is not user-defined nor is it available in strike '{strike_response.name}'"
+            )
 
-    run = client.start_strike_run(agent.latest_version.id, strike=strike, model=model)
+    run = client.start_strike_run(
+        agent.latest_version.id, strike=strike, model=model, user_model=user_model, group=group, context=context
+    )
     agent_config.add_run(run.id).write(directory)
-    formatted = format_run(run)
+    formatted = format_run(run, server_url=server_config.url)
 
     if not watch:
         print(formatted)
@@ -332,7 +445,7 @@ def deploy(
         while run.is_running():
             time.sleep(1)
             run = client.get_strike_run(run.id)
-            live.update(format_run(run))
+            live.update(format_run(run, server_url=server_config.url))
 
 
 @cli.command(help="List available models for the current (or specified) strike")
@@ -343,16 +456,24 @@ def models(
     ] = pathlib.Path("."),
     strike: t.Annotated[str | None, typer.Option("--strike", "-s", help="The strike to query")] = None,
 ) -> None:
+    user_models = UserModels.read()
+    if user_models.models:
+        print("[bold]User-defined models:[/]\n")
+        print(format_user_models(user_models.models))
+        print()
+
     if strike is None:
         agent_config = AgentConfig.read(directory)
         ensure_profile(agent_config)
+        strike = agent_config.strike
 
-    strike = strike or agent_config.strike
     if strike is None:
         raise Exception("No strike specified, use -s/--strike or set the strike in the agent config")
 
     strike_response = api.create_client().get_strike(strike)
-    print(format_models(strike_response.models))
+    if user_models.models:
+        print("\n[bold]Dreadnode-provided models:[/]\n")
+    print(format_strike_models(strike_response.models))
 
 
 @cli.command(help="List available strikes")
@@ -379,6 +500,9 @@ def latest(
     agent_config = AgentConfig.read(directory)
     ensure_profile(agent_config)
 
+    user_config = UserConfig.read()
+    server_config = user_config.get_server_config()
+
     active_link = agent_config.active_link
     if not active_link.runs:
         print(":exclamation: No runs yet, use [bold]dreadnode agent deploy[/]")
@@ -390,7 +514,7 @@ def latest(
     if raw:
         print(run.model_dump(mode="json"))
     else:
-        print(format_run(run, verbose=verbose, include_logs=logs))
+        print(format_run(run, verbose=verbose, include_logs=logs, server_url=server_config.url))
 
 
 @cli.command(help="Show the status of the active agent")
@@ -480,7 +604,7 @@ def links(
     print(table)
 
 
-@cli.command(help="Switch to a different agent link")
+@cli.command(help="Switch to a different agent link", no_args_is_help=True)
 @pretty_cli
 def switch(
     agent_or_profile: t.Annotated[str, typer.Argument(help="Agent key/id or profile name")],
@@ -502,7 +626,15 @@ def switch(
     print(f":exclamation: '{agent_or_profile}' not found, use [bold]dreadnode agent links[/]")
 
 
-@cli.command(help="Clone a github repository")
+@cli.command(help="List strike run groups")
+@pretty_cli
+def run_groups() -> None:
+    client = api.create_client()
+    groups = client.list_strike_run_groups()
+    print(format_run_groups(groups))
+
+
+@cli.command(help="Clone a github repository", no_args_is_help=True)
 @pretty_cli
 def clone(
     repo: t.Annotated[str, typer.Argument(help="Repository name or URL")],
@@ -521,7 +653,7 @@ def clone(
         shutil.rmtree(target)
 
     # Check if the repo is accessible
-    if repo_exists(github_repo):
+    if github_repo.exists:
         temp_dir = download_and_unzip_archive(github_repo.zip_url)
 
     # This could be a private repo that the user can access
