@@ -293,6 +293,16 @@ class Client:
         env: dict[str, str]
         name: str | None
 
+    class StrikeMetricPoint(BaseModel):
+        timestamp: datetime
+        value: float
+        metadata: dict[str, t.Any]
+
+    class StrikeMetric(BaseModel):
+        type: str
+        description: str | None
+        points: "list[Client.StrikeMetricPoint]"
+
     class StrikeAgentVersion(BaseModel):
         id: UUID
         created_at: datetime
@@ -337,7 +347,6 @@ class Client:
         data: dict[str, t.Any]
 
     class _StrikeRunZone(BaseModel):
-        id: UUID
         key: str
         status: "Client.StrikeRunZoneStatus"
         start: datetime | None
@@ -351,9 +360,16 @@ class Client:
         container_logs: dict[str, str]
         outputs: list["Client.StrikeRunOutput"]
         inferences: list[dict[str, t.Any]]
+        metrics: dict[str, "Client.StrikeMetric"]
+
+    class StrikeRunContext(BaseModel):
+        environment: dict[str, str] | None = None
+        parameters: dict[str, t.Any] | None = None
+        command: str | None = None
 
     class _StrikeRun(BaseModel):
         id: UUID
+        key: str
         strike_id: UUID
         strike_key: str
         strike_name: str
@@ -365,9 +381,13 @@ class Client:
         agent_name: str | None = None
         agent_revision: int
         agent_version: "Client.StrikeAgentVersion"
+        context: "Client.StrikeRunContext | None" = None
         status: "Client.StrikeRunStatus"
         start: datetime | None
         end: datetime | None
+        group_id: UUID | None
+        group_key: str | None
+        group_name: str | None
 
         def is_running(self) -> bool:
             return self.status in ["pending", "deploying", "running"]
@@ -378,6 +398,20 @@ class Client:
     class StrikeRunResponse(_StrikeRun):
         zones: list["Client.StrikeRunZone"]
 
+    class UserModel(BaseModel):
+        key: str
+        generator_id: str
+        api_key: str
+
+    class StrikeRunGroupResponse(BaseModel):
+        id: UUID
+        key: str
+        name: str
+        description: str | None
+        created_at: datetime
+        updated_at: datetime
+        run_count: int
+
     def get_strike(self, strike: str) -> StrikeResponse:
         response = self.request("GET", f"/api/strikes/{strike}")
         return self.StrikeResponse(**response.json())
@@ -431,7 +465,14 @@ class Client:
         return self.StrikeAgentResponse(**response.json())
 
     def start_strike_run(
-        self, agent_version_id: UUID, *, model: str | None = None, strike: UUID | str | None = None
+        self,
+        agent_version_id: UUID,
+        *,
+        model: str | None = None,
+        user_model: UserModel | None = None,
+        context: StrikeRunContext | None = None,
+        strike: UUID | str | None = None,
+        group: UUID | str | None = None,
     ) -> StrikeRunResponse:
         response = self.request(
             "POST",
@@ -439,7 +480,10 @@ class Client:
             json_data={
                 "agent_version_id": str(agent_version_id),
                 "model": model,
+                "user_model": user_model.model_dump(mode="json") if user_model else None,
                 "strike": str(strike) if strike else None,
+                "group": str(group) if group else None,
+                "context": context.model_dump(mode="json") if context else None,
             },
         )
         return self.StrikeRunResponse(**response.json())
@@ -454,6 +498,10 @@ class Client:
         )
         return [self.StrikeRunSummaryResponse(**run) for run in response.json()]
 
+    def list_strike_run_groups(self) -> list[StrikeRunGroupResponse]:
+        response = self.request("GET", "/api/strikes/groups")
+        return [self.StrikeRunGroupResponse(**group) for group in response.json()]
+
 
 def create_client(*, profile: str | None = None) -> Client:
     """Create an authenticated API client using stored configuration data."""
